// FILE: src/utilities/LayoutUtilities.js
var LayoutUtilities = {};
LayoutUtilities.createDiffDataStructure = function createDiffDataStructure(container, widget, clearFields){
    var containerDiff = {};
    var createWidgetsDataStructure = function createWidgetsDataStructure(widgetID, widgets, containerDiff){
        containerDiff.widgets = [];
        createWidgetsObjects(containerDiff.widgets, widgets, widgetID);
        if(Util.isArray(widgets) && widgets.length > 0){
            widgets.forEach(function(eachWidget){
                createWidgetsDataStructure(widgetID, eachWidget.widgets, getWidgetObjectByID(containerDiff.widgets, eachWidget.id));
            });
        }
    };

    var createWidgetsObjects = function createWidgetsObjects(widgetsDiff, widgetsArray, widgetID){
        if(Util.isArray(widgetsArray) && widgetsArray.length > 0){
            widgetsArray.forEach(function(eachWidget){
                var properties = {
                    "id": eachWidget.id
                };
                if(eachWidget.id == widgetID){
                    properties.fields = clearFields ? [] : eachWidget.fields;
                }
                widgetsDiff.push(properties);
            });
        }
    };

    var getWidgetObjectByID = function getWidgetObjectByID(widgets, widgetId){
        return widgets.filter(function(eachWidgetObject){
            return eachWidgetObject.id == widgetId;
        }).shift();
    };

    createWidgetsDataStructure(widget.id, container.widgets, containerDiff);
    return containerDiff;
};


// FILE: src/services/ZhLayoutPreferences.js
/**
 * Layout
 * @typedef {Object} Layout
 *
 * @since 3.0.0
 *
 * @property {String} ID                  Layout unique identifier
 * @property {String} USER_ID             User identifier that created the layout
 * @property {String} ORGANIZATION_ID     Organization the user belongs to
 * @property {String} LABEL_TO_PREFERENCE Layout label
 * @property {String} WIDGET_ID           Widget identifier what the layout belongs to
 * @property {String} LAYOUT_CHANGES      Widget changes saved on the layout
 * @property {String} LAST_MODIFIED_DATE  Last date the layout was modified
 */

/**
 * Service to handle preferences storage
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}              ZhPreferences   Service to handle preferences storage
 * @param {TemplateManager}            templateManager Service to handle template
 * @param {EventAggregator}            eventAggregator Service to use project events
 * @param {ContainerMerger}            containerMerger Service to handle layout changes
 * @param {ZHPromise}                  ZHPromise       Service to handle promises
 * @param {EventEngine}                eventEngine     Service to use models events
 * @param {ZhUtil}                     ZhUtil          Service to provice utilities methods
 * @param {MessageService}             MessageService  Service to handle messages
 */
function ZhLayoutPreferences(ZhPreferences, templateManager, eventAggregator, containerMerger, ZHPromise, eventEngine, ZhUtil, MessageService){

    var defaultOriginalLayout = {
        'ID': 'ORIGINAL',
        'USER_ID': 0,
        'ORGANIZATION_ID': 0,
        'LAYOUT_CHANGES': '{}',
        'LABEL_TO_PREFERENCE': 'Original'
    };

    function shouldShowLayoutAction(widget){
        return Util.get(widget, ['floatingControl', 'customizationAction', 'layout']) !== false;
    }

    eventAggregator.subscribe('beforeProjectStartup', function(){
        ZhPreferences.registerRepository('zh-layout-preferences#layout');

        templateManager.project.customFloatingActions.push({
            'name': 'Layout Action',
            'label': 'Layout',
            'type': 'layout',
            'order': 20,
            'icon': 'layout',
            'template': 'zh-layout-preferences#floating-layout-card.html',
            'expandable': true,
            'isVisible': shouldShowLayoutAction
        });
    });

    eventEngine.registerEvent('LayoutBeforeApply', 'beforeLayoutApply');
    eventEngine.registerEvent('LayoutAfterApply', 'afterLayoutApply');

    var layoutRegisteredDataSources = {};

    /**
     * Register layouts datasource
     *
     * @since 3.0.0
     *
     * @param {String} widgetID         Widget identifier
     * @param {Object} datasource       Datasource that contains the layouts
     *
     */
    this.registerLayoutDataSource = function(widgetID, dataSource){
        if(widgetID){
            layoutRegisteredDataSources[widgetID] = dataSource;
        }

        return layoutRegisteredDataSources;
    };

    /**
     * Update dataSource with local saved layouts
     *
     * @since 3.0.0
     *
     * @param  {String}     widgetID   Widget identifier
     * @param  {DataSource} dataSource DataSource that contains the loaded layouts
     *
     * @return {Promise}               Promise resolved after the layouts are loaded into the dataSource
     */
    this.updateLayouts = function(widgetID, dataSource){
        if(!dataSource){
            dataSource = layoutRegisteredDataSources[widgetID];
        }

        return this.getWidgetLayouts(widgetID).then(function(layouts){
            return layouts;
        }).then(function(layouts){
            return this.addIsDefaultLayoutToLayouts(layouts, widgetID).then(function(layouts){
                dataSource.data = layouts;
            });
        }.bind(this));
    };

    /**
     * Add property to identify if the layout is selected as default
     *
     * @since 3.0.0
     *
     * @param {Layout[]} layouts  List of layouts to update the default layout property
     * @param {String}   widgetID Widget to get the default properties
     */
    this.addIsDefaultLayoutToLayouts = function(layouts, widgetID){
        return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences){
            var defaultLayout = defaultPreferences.DEFAULT_LAYOUT;
            return layouts.map(addIsDefaultLayoutToLayout.bind(this, defaultLayout));
        });
    };

    function addIsDefaultLayoutToLayout(defaultLayout, layout){
        layout.DEFAULT_LAYOUT = layout.ID === defaultLayout;
        return layout;
    }

    /**
     * Retrieve local cached layouts for specified widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetID   Widget identifier
     *
     * @return {Promise<Layout[]>} Layout preferences
     */
    this.getWidgetLayouts = function(widgetID){
        return ZhPreferences.getWidgetPreferences('layout', widgetID);
    };

    /**
     * Retrieve local cached layouts
     *
     * @since 3.0.0
     *
     * @param  {Query} query       Conditions to filter the preferences
     *
     * @return {Promise<Layout[]>} Layout preferences
     */
    this.getLayouts = function(query){
        return ZhPreferences.getPreferences('layout', query);
    };

    /**
     * Persist layouts
     *
     * @since 3.0.0
     *
     * @param  {Layout[]} layouts  Layouts to persist
     *
     * @return {Promise<Layout[]>} List of persisted layouts
     */
    this.saveLayouts = function(layouts){
        var cloneLayouts = layouts.map(function(layout){ return ZhUtil.clearRow(layout); });
        return ZhPreferences.savePreferences('layout', cloneLayouts).then(function(response){
            cloneLayouts.forEach(function(clone, index){
                layouts[index].ID = clone.ID;
                layouts[index].__is_new = clone.__is_new;
                layouts[index].changes = clone.changed;
            });

            return ZHPromise.when(response);
        });
    };

    /**
     * Delete layouts
     *
     * @since 3.0.0
     *
     * @param  {Layout[]} layouts Layouts to delete
     *
     * @return {Promise<Layout[]>} List of deleted layouts
     */
    this.deleteLayouts = function(layouts){
        var cloneLayouts = layouts.map(function(layout){ return ZhUtil.clearRow(layout); });
        return ZhPreferences.deletePreferences('layout', cloneLayouts);
    };

    function triggerBeforeLayoutApplyEvent(widget, layout){
        if(widget.beforeLayoutApply){
            widget.beforeLayoutApply({
                'layout': layout
            });
        }
    }

    function triggerAfterLayoutApplyEvent(widget, layout){
        if(widget.afterLayoutApply){
            widget.afterLayoutApply({
                'layout': layout
            });
        }
    }

    var appliedPreference = {ID: 'ORIGINAL'};

    /**
     * Apply original layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget Widget to apply the original layout on
     *
     * @return {Promise}       Promise resolved after the original layout is applied
     */
    this.applyOriginalLayout = function(widget){
        triggerBeforeLayoutApplyEvent(widget, defaultOriginalLayout);

        containerMerger.restore(widget, 'ORIGINAL');
        widget.redraw();
        widget.setAppliedLayout(defaultOriginalLayout);
        appliedPreference = defaultOriginalLayout;

        triggerAfterLayoutApplyEvent(widget, defaultOriginalLayout);

        return ZHPromise.when();
    };

    /**
     * Apply layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget     Widget to apply the layout on
     * @param  {Layout} preference Layout preference to be applied
     *
     * @return {Promise}           Promise that is resolved after the layout is applied
     */
    this.applyLayout = function(widget, preference){
        triggerBeforeLayoutApplyEvent(widget, preference);

        if(widget.backupPDFOrientation === undefined){
            widget.backupPDFOrientation = widget.reportOrientation || 'PORTRAIT';
        }
        widget.reportOrientation = preference.PDF_REPORT_ORIENTATION;

        var layoutChanges = prepareLayout(JSON.parse(preference.LAYOUT_CHANGES));
        containerMerger.merge(templateManager.container, layoutChanges, appliedPreference.ID);

        widget.redraw(true);

        widget.setAppliedLayout(preference);
        appliedPreference = preference;

        triggerAfterLayoutApplyEvent(widget, preference);

        return ZHPromise.when();
    };

    var propertyToBeVerified = {
        container: ['widgets'],
        widget: ['widgets', 'fields']
    };

    function prepareLayout(objToBeMerged, baseObject){
        baseObject = baseObject || templateManager.container;
        var childProperties = [];

        if(baseObject instanceof Container){
            childProperties = propertyToBeVerified.container;
        } else if(baseObject instanceof Widget){
            childProperties = propertyToBeVerified.widget;
        }

        childProperties.forEach(function prepareChildLayout(propertyName){
            var childBaseObj = baseObject[propertyName];
            var childObjToBeMerged = objToBeMerged[propertyName];
            if(childObjToBeMerged){
                if(childBaseObj && childBaseObj.length){
                    var existentIds = childBaseObj.map(function returnObjId(obj){
                        return obj.id;
                    });
                    objToBeMerged[propertyName] = childObjToBeMerged.filter(function filterChilds(obj){
                        var objIndex = existentIds.indexOf(obj.id);
                        if(objIndex != -1){
                            prepareLayout(obj, childBaseObj[objIndex]);
                        }
                        return ~objIndex;
                    });
                } else {
                    objToBeMerged[propertyName] = [];
                }
            }
        });

        return objToBeMerged;
    }

    /**
     * Export layout file
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout  Layout to export
     *
     * @return {Promise<Layout>} Layout exported
     */
    this.downloadPreferenceFile = function(layout){
        var fileName = layout.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/ig,'').replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, function(match, index){
            if(+match === 0) return "";
            return index == 0 ? match.toLowerCase() : match.toUpperCase();
        }) + '.layout';

        var layoutToExport = {
            'ID': layout.ID,
            'LABEL_TO_PREFERENCE': layout.LABEL_TO_PREFERENCE,
            'WIDGET_ID': layout.WIDGET_ID,
            'PDF_REPORT_ORIENTATION': layout.PDF_REPORT_ORIENTATION,
            'LAYOUT_CHANGES': JSON.parse(layout.LAYOUT_CHANGES),
            'LAST_MODIFIED_DATE': layout.LAST_MODIFIED_DATE
        };

        return ZhPreferences.downloadPreferenceFile(layoutToExport, fileName);
    };

    /**
     * Validade layout to import
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout              Layout to import
     * @param  {Widget} configWidget        Widget to validate
     * @param  {Object[]} layoutDataSource  Existing layouts data (optional)
     *
     * @return {Boolean} Layout valid
     */
    this.validateLayoutImport = function(layout, configWidget, layoutDataSource){
        layoutDataSource = layoutDataSource || configWidget.dataSource.data;

        if(configWidget.listWidget.id != layout.WIDGET_ID){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_widget_id');
            return false;
        }

        var layoutsWithSameId = layoutDataSource.filter(function(row){
            return row.ID == layout.ID;
        });

        if(layoutsWithSameId.length > 0){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_id_violation');
            return false;
        }

        var layoutsWithSameLabel = layoutDataSource.filter(function(row){
            return row.LABEL_TO_PREFERENCE == layout.LABEL_TO_PREFERENCE;
        });

        if(layoutsWithSameLabel.length > 0){
            MessageService.showMessageByCode('zh-layout-preferences#import_error_name_violation');
            return false;
        }

        return true;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutPreferences', ZhLayoutPreferences);
});

// FILE: src/controllers/ZhLayoutWidgetsBuilder.js
/**
 * Service to create widgets to handle layout
 *
 * @class
 * @since 3.0.0
 *
 * @param {RestEngine}      restEngine      Service to make http requests
 * @param {TemplateManager} templateManager Service to handle templates
 * @param {MetaDataFactory} metaDataFactory Service to factory models from metadata
 */
var ZhLayoutWidgetsBuilder = function(restEngine, templateManager, metaDataFactory){

    function getWidgetJSON(widgetName){
        var widgetPath = 'zh-layout-preferences#json/widgets/'+widgetName+'.json';
        return restEngine.requestMetaData(widgetPath).then(function(response){
            return response.data;
        });
    }

    /**
     * Retrieve a factored widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetName Widget name
     *
     * @return {Widget}            Factored widget
     */
    this.factoryWidget = function(widgetName){
        return getWidgetJSON(widgetName).then(function(widget){
            return metaDataFactory.widgetFactory(widget, templateManager.container);
        });
    };

    /**
     * Retrieve factored layout configuration widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Layout configuration widget
     */
    this.factoryLayoutConfigurationWidget = function(){
        return this.factoryWidget('config');
    };

    /**
     * Retrieve factored create layout widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Create layout widget
     */
    this.factoryCreateLayoutWidget = function(){
        return this.factoryWidget('createLayout');
    };

    /**
     * Retrieve factored import layout widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Import layout widget
     */
    this.factoryImportLayoutWidget = function(){
        return this.factoryWidget('importLayout');
    };

    /**
     * Retrieve factored build layout options widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Build layout options widget
     */
    this.factoryBuildLayoutOptionsWidget = function(){
        return this.factoryWidget('createLayoutOptions');
    };

    /**
     * Retrieve factored build layout field options widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Build layout field options widget
     */
    this.factoryBuildLayoutFieldOptionsWidget = function(){
        return this.factoryWidget('createLayoutFieldOptions');
    };

};

ZeedhiController.controller('ZhLayoutWidgetsBuilder', ZhLayoutWidgetsBuilder);

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutWidgetsBuilder', ZhLayoutWidgetsBuilder);
});

// FILE: src/controllers/ZhLayoutCreateLayoutOptionsController.js
/**
 * Controller to manipulate create layout options widget
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                             MessageService                             Service to show messages
 * @param {ZhLayoutPreferences}                        ZhLayoutPreferences                        Service to handle layout preferences methods
 * @param {ScreenService}                              ScreenService                              Service to manipulate screen
 * @param {ZhLayoutWidgetsBuilder}                     ZhLayoutWidgetsBuilder                     Service to create widgets to handle layouts
 * @param {ZHPromise}                                  ZHPromise              Service to handle promises
 */
var ZhLayoutCreateLayoutOptionsController = function(MessageService, ZhLayoutPreferences, ScreenService, ZhLayoutWidgetsBuilder, metaDataFactory, ZHPromise){

    var fieldDefaultProperties = {
        isVisible: true,
        showOnList: true,
        visibility: true,
        grouped: false,
        fixed: false
    };

    var createLayoutOptionsWidgetPromise = null;

    function removeConflictingProperties(field){
        var conflictingProperties = ['selectWidget', 'inlineEditable'];
        conflictingProperties.forEach(function(property){
            delete field[property];
        });
    }

    function factoryFields(parentWidget, gridFields){
        return gridFields.filter(function(field){
            return !field.skipLayout && !!field.label;
        }).map(function(field){
            var factoredField = metaDataFactory.fieldFactory(field, parentWidget);
            removeConflictingProperties(factoredField);
            return factoredField;
        });
    }

    /**
     * Retrieve widget that handles create layout options
     *
     * @since 3.0.0
     *
     * @param  {Layout} preference   Layout to be changed
     * @param  {Widget} configWidget Widget that handles layouts configuration
     *
     * @return {Widget}              Widget that handles create layout options
     */
    this.getCreateLayoutOptionsWidget = function(preference, configWidget){
        if(createLayoutOptionsWidgetPromise === null){
            createLayoutOptionsWidgetPromise = ZhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget();
        }

        return createLayoutOptionsWidgetPromise.then(function(layoutOptionsWidget){
            var label = preference.LABEL_TO_PREFERENCE;

            var listWidget = configWidget.listWidget;
            layoutOptionsWidget.listWidget = listWidget;

            layoutOptionsWidget.preference = preference;
            layoutOptionsWidget.label = ScreenService.i18n('Layout');
            if(label){
                layoutOptionsWidget.label = layoutOptionsWidget.label + ' - ' + label;
            }
            layoutOptionsWidget.layoutName = label;
            layoutOptionsWidget.reportTitle = listWidget.label;
            layoutOptionsWidget.reportOrientation = preference.PDF_REPORT_ORIENTATION;

            // get only the first two rows from listWidget
            layoutOptionsWidget.dataSource.data = Util.clone(listWidget.dataSource.data.slice(0,2));

            // get fields from listWidget
            layoutOptionsWidget.fields = factoryFields(layoutOptionsWidget, listWidget.fields);

            var layoutChanges = JSON.parse(preference.LAYOUT_CHANGES);
            var widgetDifferences = getEditingWidgetPath(layoutChanges.widgets, listWidget);

            layoutOptionsWidget.fields = layoutOptionsWidget.fields.map(function(field, index){
                delete field.__isSelected;
                delete field.__leftActions;
                delete field.__notSelectable;
                delete field.__rightActions;

                if(!Util.isEmptyOrBlank(widgetDifferences.fields)){
                    var fieldPreference = filterObject(widgetDifferences.fields, field.id, "id");
                    if(fieldPreference.obj && Object.keys(fieldPreference.obj).length){
                        return Util.extend(field, fieldPreference.obj);
                    }
                }

                return Util.extend(fieldDefaultProperties, field, true);
            });

            return layoutOptionsWidget;
        });
    };

    /**
     * Save and apply layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} createLayoutOptionsWidget Widget that handles layout options
     *
     * @return {Promise}                          Promise resolved when the layout is applied
     */
    this.saveAndApplyLayout = function(createLayoutOptionsWidget){
        if(createLayoutOptionsWidget.gridConfigWidget.isValid()){
            var configWidget = createLayoutOptionsWidget.createNewLayoutWidget.configWidget;
            var listWidget = configWidget.listWidget;

            var fields = getFieldEditingWidgetAsDataSource(createLayoutOptionsWidget);
            var layout = createLayoutOptionsWidget.preference;

            var layoutChanges = JSON.parse(layout.LAYOUT_CHANGES);
            var editingWidget = getEditingWidgetPath(layoutChanges.widgets, listWidget);
            editingWidget.fields = fields;

            layout.PDF_REPORT_ORIENTATION = createLayoutOptionsWidget.reportOrientation;
            layout.LABEL_TO_PREFERENCE = createLayoutOptionsWidget.layoutName;
            layout.LAYOUT_CHANGES = JSON.stringify(layoutChanges);

            return ZhLayoutPreferences.saveLayouts([layout]).then(function(layouts){
                return ZhLayoutPreferences.applyLayout(listWidget, layouts[0]);
            }).then(function(){
                ScreenService.closePopup(true);
                ZhLayoutPreferences.updateLayouts(listWidget.id, configWidget.dataSource);
                listWidget.activate();
            }).catch(function(error){
                MessageService.showMessageByCode('zh-layout-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        } else {
            return ZHPromise.reject("Invalid params");
        }
    };

    /**
     * Change field visibility
     *
     * @since 3.0.0
     *
     * @param {Field} field Field to change the visibility
     */
    this.updateFieldVisibility = function(field){
        field.isVisible = true;
        field.showOnList = field.isVisibleOnReport = field.visibility;
    };

    function getFieldEditingWidgetAsDataSource(listWidget){
        return listWidget.fields.map(function(field){
            var data = {
                'id': field.id,
                'name': field.name,
                'label': field.label,
                'labelOnReport': field.labelOnReport || null,
                'expression': field.expression || null,
                'align': field.align || null,
                'isVisible': field.isVisible,
                'showOnList': field.showOnList,
                'minWidth': field.gridColumnSize || null,
                'maxWidth': field.gridColumnSize || null,
                'gridColumnSize': field.gridColumnSize || null,
                'reportColumnSize': field.reportColumnSize || null,
                'isVisibleOnReport': field.isVisibleOnReport,
                'sequenceInList': field.sequenceInList,
                'visibility': field.isVisible && field.showOnList,
                'grouped': field.grouped || false,
                'groupOpened': field.groupOpened,
                'fixed': field.fixed || false,
                'skipLayout': field.skipLayout || false
            };
            return data;
        }).filter(function(field){
            return !field.skipLayout;
        });
    }

    function getEditingWidgetPath(widgetsArray, listWidget){
        var foundWidget = getWidgetObjectByID(widgetsArray, listWidget.id);
        if(!foundWidget){
            widgetsArray.some(function(eachWidget){
                foundWidget = getEditingWidgetPath(eachWidget.widgets, listWidget);
                return foundWidget;
            });
        }
        return foundWidget;
    }

    function filterObject(obj, value, key){
        var filtered = obj.filter(function(preference){
            return preference[key] == value;
        });

        var toReturn = {};
        toReturn.index = obj.indexOf(filtered[0]);
        toReturn.obj = filtered[0];
        return toReturn;
    }

    function getWidgetObjectByID(widgets, widgetID){
        return widgets.filter(function(eachWidgetObject){
            return eachWidgetObject.id == widgetID;
        }).shift();
    }

};

ZeedhiController.controller('ZhLayoutCreateLayoutOptionsController', ZhLayoutCreateLayoutOptionsController);

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutCreateLayoutOptionsController', ZhLayoutCreateLayoutOptionsController);
});

// FILE: src/controllers/ZhLayoutImportLayoutController.js
/**
 * Import layout widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                        MessageService                           Service to show messages
 * @param {ZhLayoutPreferences}                   ZhLayoutPreferences                      Service to handle layout preferences methods
 * @param {ZhPreferences}                         ZhPreferences                            Service to handle preferences persistance
 * @param {ZhLayoutWidgetsBuilder}                ZhLayoutWidgetsBuilder Service to create widgets to handle layouts
 * @param {ZHPromise}                             ZHPromise                                Service to handle promises
 * @param {ScreenService}                         ScreenService                            Service to manipulate screen
 */
var ZhLayoutImportLayoutController = function(MessageService, ZhLayoutPreferences, ZhPreferences, ZhLayoutWidgetsBuilder, ZHPromise, ScreenService){

    var importLayoutWidgetPromise = null;

    /**
     * Retrieve widget to import layout
     *
     * @since 3.0.0
     *
     * @return {Promise<Widget>}
     */
    this.getImportLayoutWidget = function(){
        if(importLayoutWidgetPromise === null){
            importLayoutWidgetPromise = ZhLayoutWidgetsBuilder.factoryImportLayoutWidget();
        }

        return importLayoutWidgetPromise;
    };

    /**
     * Import layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} importLayoutWidget Widget with the layout configuration
     *
     * @return {Promise}                      Promise resolved after the layout is imported
     */
    this.importPreference = function(importLayoutWidget){
        if(!importLayoutWidget.isValid()){
            return ZHPromise.reject({err: 'WIDGET_INVALID'});
        }

        var layoutFileData = importLayoutWidget.fields[0].value()[0].b64File;
        layoutFileData = layoutFileData.split('base64,')[1];
        layoutFileData = window.atob(layoutFileData);
        layoutFileData = decodeURIComponent(escape(layoutFileData));

        var layout = JSON.parse(layoutFileData);

        var configWidget = importLayoutWidget.configWidget;
        var listWidget = configWidget.listWidget;

        if(!ZhLayoutPreferences.validateLayoutImport(layout, configWidget)){
            return ZHPromise.reject({err: 'LAYOUT_FILE_INVALID'});
        }

        layout.__is_new = true;
        layout.__generate_id = false;
        layout.LAYOUT_CHANGES = JSON.stringify(layout.LAYOUT_CHANGES);

        return ZhPreferences.getUserID().then(function(userID){
            layout.USER_ID = userID;

            return ZhLayoutPreferences.saveLayouts([layout]).then(function(){
                ScreenService.closePopup();
                ZhLayoutPreferences.updateLayouts(listWidget.id, configWidget.dataSource);
                configWidget.refreshWidget();
            }).catch(function(error){
                MessageService.showMessageByCode('zh-layout-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        });
    };
};

ZeedhiController.controller('ZhLayoutImportLayoutController', ZhLayoutImportLayoutController);

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutImportLayoutController', ZhLayoutImportLayoutController);
});

// FILE: src/controllers/ZhLayoutConfigController.js
/**
 * Layout configuration widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}                         ZhPreferences                         Service to handle preferences persistance
 * @param {ZhLayoutPreferences}                   ZhLayoutPreferences                   Service to handle layout preferences methods
 * @param {ZhLayoutWidgetsBuilder}                ZhLayoutWidgetsBuilder                Service to create widgets to handle layouts
 * @param {ScreenService}                         ScreenService                         Service to manipulate screen
 * @param {MessageService}                        MessageService                        Service to show messages
 * @param {ZHPromise}                             ZHPromise                             Service to handle promises
 * @param {$timeout}                              $timeout                              Service to handle timeouts
 * @param {ZhLayoutCreateLayoutOptionsController} ZhLayoutCreateLayoutOptionsController Controller to manipulate create layout options widget
 * @param {ZhLayoutImportLayoutController}        ZhLayoutImportLayoutController        Controller to manipulate import layout widget
 */
var ZhLayoutConfigController = function(ZhPreferences, ZhLayoutPreferences, ZhLayoutWidgetsBuilder, ScreenService, MessageService, ZHPromise, $timeout, ZhLayoutCreateLayoutOptionsController, ZhLayoutImportLayoutController){

    var configWidgetPromise = null;

    /**
     * Retrieve widget that handles layouts configuration
     *
     * @since 3.0.0
     *
     * @param  {Widget}     listWidget Widget that is being used to create layouts for
     * @param  {DataSource} dataSource DataSource to use on config widget
     *
     * @return {Promise<Widget>}       Widget that handles layouts configuration
     */
    this.getConfigWidget = function(listWidget, dataSource){
        if(configWidgetPromise === null){
            configWidgetPromise = ZhLayoutWidgetsBuilder.factoryLayoutConfigurationWidget();
        }

        var userID = ZhPreferences.getUserID();

        return ZHPromise.all([userID, configWidgetPromise]).then(function(args){
            var configWidget = args[1];

            configWidget.listWidget = listWidget;
            configWidget.dataSourceFilter = [
                {'name': 'WIDGET_ID', 'operator': '=', 'value': listWidget.id}
            ];
            dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == (listWidget.appliedLayout ? listWidget.appliedLayout.ID : 'ORIGINAL');
            });
            configWidget.dataSource = dataSource;

            return configWidget;
        });
    };

    /**
     * Open widget to create a new layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layout configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.openCreateLayoutWidget = function(configWidget){
        var listWidget = configWidget.listWidget;
        var pdfReportOrientation = listWidget.reportOrientation || "PORTRAIT";

        return ZhPreferences.getUserID().then(function(userID){
            return {
                '__is_new': true,
                'ID': null,
                'WIDGET_ID': listWidget.id,
                'USER_ID': userID,
                'LAYOUT_CHANGES': JSON.stringify(LayoutUtilities.createDiffDataStructure(listWidget.container, listWidget, true)),
                'PDF_REPORT_ORIENTATION': pdfReportOrientation,
                'LAST_MODIFIED_DATE': null
            };
        }).then(function(preference){
            return ZhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(createLayoutOptionsWidget){
                createLayoutOptionsWidget.createNewLayoutWidget = {'configWidget': configWidget};

                $timeout(function(){
                    ScreenService.openPopup(createLayoutOptionsWidget);
                });
            });
        });

    };

    /**
     * Open popup to edit a saved layout
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout   Layout to edit
     *
     * @return {Promise<Widget>} Widget used to edit the layout
     */
    this.editLayout = function(configWidget, layout){
        if(layout.TYPE === 'F'){
            MessageService.alertNotificationByCode('zh-layout-preferences#cannot_edit_fixed_layout');
            return ZHPromise.when();
        }
        return ZhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(layout, configWidget).then(function(createLayoutOptionsWidget){
            createLayoutOptionsWidget.createNewLayoutWidget = {'configWidget': configWidget};

            $timeout(function(){
                ScreenService.openPopup(createLayoutOptionsWidget);
            });
        });
    };

    /**
     * Delete layouts
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that contains the layouts to delete
     *
     * @return {Promise}             Promise resolved after layouts are deleted
     */
    this.deleteLayouts = function(configWidget){
        var layoutsToDelete = configWidget.getCheckedRows();

        var defaultLayout = layoutsToDelete.filter(function(layout){
            return layout.DEFAULT_LAYOUT;
        });
        if(defaultLayout.length){
            MessageService.showMessageByCode('zh-layout-preferences#delete_error_default');

            return;
        }

        return MessageService.confirmMessageByCode('zh-layout-preferences#delete', {
            'NR_OF_LAYOUTS': layoutsToDelete.length
        }).then(function(){
            return ZhLayoutPreferences.deleteLayouts(layoutsToDelete).then(function(){
                configWidget.dataSource.clearCheckedRows();
                return ZhLayoutPreferences.updateLayouts(configWidget.listWidget.id, configWidget.dataSource);
            }).catch(function(error){
                return MessageService.getMessageByCode('zh-layout-preferences#'+error.message).then(function(errorMessage) {
                    return MessageService.showMessageByCode('zh-layout-preferences#delete_error', {
                        'ERROR': ScreenService.i18n(errorMessage)
                    });
                });
            });
        });
    };

    /**
     * Set a layout as the default preference
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layouts configuration
     * @param  {Layout} layout       Layout that it's default value has changed
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateDefaultLayout = function(configWidget, layout){
        return ZhPreferences.changeDefaultPreferences(configWidget, 'DEFAULT_LAYOUT', layout);
    };

    /**
     * Apply Layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layouts configuration
     * @param  {Layout} layout       Layout to be applied
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateAppliedLayout = function(configWidget, layout){
        var listWidget = configWidget.listWidget;

        return ZhLayoutPreferences.applyLayout(listWidget, layout).then(function(){
            configWidget.dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == layout.ID;
            });
        });
    };

    /**
     * Download preference as file
     *
     * @since 3.0.0
     *
     * @param  {Layout} layout       Layout to export
     *
     * @return {Promise<Layout>}     Layout exportado
     */
    this.downloadPreferenceFile = function(layout){
        return ZhLayoutPreferences.downloadPreferenceFile(layout);
    };

    /**
     * Open widget to import a new layout from file
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles layout configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.importPreferenceFile = function(configWidget){
        return ZhLayoutImportLayoutController.getImportLayoutWidget().then(function(importLayoutWidget){
            importLayoutWidget.newRow();
            importLayoutWidget.configWidget = configWidget;
            return ScreenService.openPopup(importLayoutWidget);
        });
    };

};

ZeedhiController.controller('ZhLayoutConfigController', ZhLayoutConfigController);

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutConfigController', ZhLayoutConfigController);
});

// FILE: src/controllers/ZhLayoutFloatingCardController.js
/**
 * Floating window controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {$scope}              $scope              Floating window scope
 * @param {ApplicationContext}  ApplicationContext  Service to retrieve Zeedhi services
 * @param {Query}               Query               Service to build queries
 * @param {ScreenService}       ScreenService       Service to handle screen
 * @param {metaDataFactory}     metaDataFactory     Service to factory models from metadata
 */
var ZhLayoutFloatingCardController = function($scope, ApplicationContext, Query, ScreenService, metaDataFactory){

    var widget = $scope.widget;
    $scope.appliedPreference = null;

    var ZhLayoutPreferences = ApplicationContext.ZhLayoutPreferences;
    var ZhPreferences = ApplicationContext.ZhPreferences;
    var ZhLayoutConfigController = ApplicationContext.ZhLayoutConfigController;

    var fieldDefaultProperties = {
        isVisible: true,
        showOnList: true,
        visibility: true,
        grouped: false,
        fixed: false
    };

    var dataSource = metaDataFactory.factoryDataSource({
        'name': 'zh-layout-preferences#/layout',
        'localStorage': false,
        'lazyLoad': true,
        'rest': true,
        'data': []
    });

    dataSource.afterLoadDataSource(function(){
        return ZhLayoutPreferences.addIsDefaultLayoutToLayouts(dataSource.data, widget.id).then(function(layouts){
            dataSource.data = layouts;
        });
    });

    $scope.customizationCard = {
        'label': 'Layouts',
        'type': 'layout',
        'openLayoutConfiguration': openLayoutConfiguration,
        'refreshLayouts': refreshLayouts,
        'applyOriginalLayout': applyOriginalLayout,
        'applyLayout': applyPreference,
        'dataSource': dataSource
    };

    ZhLayoutPreferences.registerLayoutDataSource(widget.id, $scope.customizationCard.dataSource);

    widget.setAppliedLayout = setAppliedLayout;

    $scope.$on('$destroy', function() {
        delete dataSource.afterLoadDataSourcelistener;
    });

    function setAppliedLayout(layout){
        $scope.appliedPreference = layout;
        widget.appliedLayout = layout;
    }

    function openLayoutConfiguration(){
        var dataSource = $scope.customizationCard.dataSource;
        return ZhLayoutConfigController.getConfigWidget(widget, dataSource).then(function(configWidget){
            return ScreenService.openPopup(configWidget);
        });
    }

    function refreshLayouts(){
        return ZhPreferences.getUserID().then(function(){
            var query = Query.build()
                .where('WIDGET_ID').equals(widget.id);

            return $scope.customizationCard.dataSource.load(query.where());
        });
    }

    function applyOriginalLayout(){
        return ZhLayoutPreferences.applyOriginalLayout(widget);
    }

    function applyPreference(layout){
        return ZhLayoutPreferences.applyLayout(widget, layout);
    }

    function applyDefaultLayout(){
        ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(defaultPreference){
            if(defaultPreference.defaultLayout){
                applyPreference(defaultPreference.defaultLayout);
            } else {
                applyOriginalLayout();
            }
        });
    }

    (function initLayoutPreferences(){
        if(ZhPreferences.validateWidget(widget)){
            if(Util.isUndefinedOrNull(widget.appliedLayout)){
                applyDefaultLayout();
            } else {
                applyPreference(widget.appliedLayout);
            }
        }
    })();

};

ZeedhiController.controller('ZhLayoutFloatingCardController', ZhLayoutFloatingCardController);

// FILE: src/directives/ZhLayoutFloatingControl.js
var ZeedhiDirectives = angular.module('ZeedhiDirectives');

ZeedhiDirectives.directive('zhLayoutFloatingControl', [ZhLayoutFloatingControl]);

function ZhLayoutFloatingControl(){

    return {
        priority: 10,
        link: function($scope){
            $scope.toggleLayoutPreferencesCard = function(e){
                var actionElement = $(e.target).closest('.float-action');
                if(!actionElement.hasClass('opened')){
                    $scope.customizationCard.refreshLayouts();
                }
                $scope.togglePreferencesCard(e);
            };
        }
    };

}

// FILE: src/Config.js
(function(){

window.templateUrls = window.templateUrls || [];
window.metadataUrls = window.metadataUrls || [];
window.serviceUrls = window.serviceUrls || [];

var PACKAGE_NAME = 'zh-layout-preferences';

var hasTemplateUrl = Boolean(window.templateUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

var hasMetadataUrl = Boolean(window.metadataUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

var hasServiceUrl = Boolean(window.serviceUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

if(!hasTemplateUrl){
    window.templateUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-layout-preferences/dist/templates/'
    });
}

if(!hasMetadataUrl){
    window.metadataUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-layout-preferences/dist/assets/'
    });
}

if(!hasServiceUrl){
    window.serviceUrls.push({
        package: PACKAGE_NAME,
        baseUrl: '../backend/service/index.php/zh-preferences'
    });
}

})();
